/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
function CommonSortObject(oTable, oSortTypes,isSplit) {

	this.sortTypes = oSortTypes || [];

	this.sortColumn = null;
	this.descending = null;
	this.splitTable = true;
	
	if(typeof(isSplit)!="undefined"){
		this.splitTable = isSplit;
	}

	var oThis = this;
	this._headerOnclick = function (e) {
		oThis.headerOnclick(e);
	};

	if (oTable) {
		this.setTable( oTable );
		this.document = oTable.ownerDocument || oTable.document;
	}
	else {
		this.document = document;
	}

	// only IE needs this
	var win = this.document.defaultView || this.document.parentWindow;
	this._onunload = function () {
		oThis.destroy();
	};
	
	if (win && typeof win.attachEvent != "undefined") {
		win.attachEvent("onunload", this._onunload);
	}
}

CommonSortObject.gecko = navigator.product == "Gecko";
CommonSortObject.msie = /msie/i.test(navigator.userAgent);
// Mozilla is faster when doing the DOM manipulations on
// an orphaned element. MSIE is not
CommonSortObject.removeBeforeSort = CommonSortObject.gecko;

CommonSortObject.prototype.onsort = function () {};

// default sort order. true -> descending, false -> ascending
CommonSortObject.prototype.defaultDescending = false;

// shared between all instances. This is intentional to allow external files
// to modify the prototype
CommonSortObject.prototype._sortTypeInfo = {};

CommonSortObject.prototype.setTable = function (oTable) {
	if ( this.tHead )
		this.uninitHeader();
	this.element = oTable;
	this.setTHead( oTable.tHead );
	this.setTBody( oTable.tBodies[0] );
};

CommonSortObject.prototype.setTHead = function (oTHead) {
	if (this.tHead && this.tHead != oTHead )
		this.uninitHeader();
	this.tHead = oTHead;
	this.initHeader( this.sortTypes );
};

CommonSortObject.prototype.setTBody = function (oTBody) {
	this.tBody = oTBody;
};

CommonSortObject.prototype.setSortTypes = function ( oSortTypes ) {
	if ( this.tHead )
		this.uninitHeader();
	this.sortTypes = oSortTypes || [];
	if ( this.tHead )
		this.initHeader( this.sortTypes );
};

// adds arrow containers and events
// also binds sort type to the header cells so that reordering columns does
// not break the sort types
CommonSortObject.prototype.initHeader = function (oSortTypes) {
    //currently , rssm header is hidden row for accessibility, so ignore the header in sorting
	if (this.splitTable) return;
	if (!this.tHead) return;
	var cells = this.tHead.rows[0].cells;
	var doc = this.tHead.ownerDocument || this.tHead.document;
	this.sortTypes = oSortTypes || [];
	var l = cells.length;
	var img, c;
	for (var i = 0; i < l; i++) {
		c = cells[i];
		if (this.sortTypes[i] != null && this.sortTypes[i] != "None") {
			img = doc.createElement("IMG");
			img.src = path + "/images/blank.png";
			c.appendChild(img);
			if (this.sortTypes[i] != null)
				c._sortType = this.sortTypes[i];
			if (typeof c.addEventListener != "undefined")
				c.addEventListener("click", this._headerOnclick, false);
			else if (typeof c.attachEvent != "undefined")
				c.attachEvent("onclick", this._headerOnclick);
			else
				c.onclick = this._headerOnclick;
		}
		else
		{
			c.setAttribute( "_sortType", oSortTypes[i] );
			c._sortType = "None";
		}
	}
	this.updateHeaderArrows();
};

// remove arrows and events
CommonSortObject.prototype.uninitHeader = function () {
	if (!this.tHead) return;
	var cells = this.tHead.rows[0].cells;
	var l = cells.length;
	var c;
	for (var i = 0; i < l; i++) {
		c = cells[i];
		if (c._sortType != null && c._sortType != "None") {
			c.removeChild(c.lastChild);
			if (typeof c.removeEventListener != "undefined")
				c.removeEventListener("click", this._headerOnclick, false);
			else if (typeof c.detachEvent != "undefined")
				c.detachEvent("onclick", this._headerOnclick);
			c._sortType = null;
			c.removeAttribute( "_sortType" );
		}
	}
};

CommonSortObject.prototype.updateHeaderArrows = function () {
	if (!this.tHead) return;
	var cells = this.tHead.rows[0].cells;
	var l = cells.length;
	var img;
	for (var i = 0; i < l; i++) {
		if (cells[i]._sortType != null && cells[i]._sortType != "None") {
			img = cells[i].lastChild;
			if (i == this.sortColumn)
				img.className = "sort-arrow " + (this.descending ? "descending" : "ascending");
			else
				img.className = "sort-arrow";
		}
	}
};

CommonSortObject.prototype.headerOnclick = function (e) {
	// find TD element
	var el = e.target || e.srcElement;
	while (el.tagName != "TD")
		el = el.parentNode;

	this.sort(CommonSortObject.msie ? CommonSortObject.getCellIndex(el) : el.cellIndex);
};

// IE returns wrong cellIndex when columns are hidden
CommonSortObject.getCellIndex = function (oTd) {
	var cells = oTd.parentNode.childNodes
	var l = cells.length;
	var i;
	for (i = 0; cells[i] != oTd && i < l; i++)
		;
	return i;
};

CommonSortObject.prototype.getSortType = function (nColumn) {
	return this.sortTypes[nColumn] || "String";
};

// only nColumn is required
// if bDescending is left out the old value is taken into account
// if sSortType is left out the sort type is found from the sortTypes array

CommonSortObject.prototype.sort = function (nColumn, bDescending, sSortType) {
	if (!this.tBody) return;
	if (sSortType == null)
		sSortType = this.getSortType(nColumn);

	// exit if None
	if (sSortType == "None")
		return;

	if (bDescending == null) {
		if (this.sortColumn != nColumn)
			this.descending = this.defaultDescending;
		else
			this.descending = !this.descending;
	}
	else
		this.descending = bDescending;

	this.sortColumn = nColumn;

	if (typeof this.onbeforesort == "function")
		this.onbeforesort();

	var f = this.getSortFunction(sSortType, nColumn);
	var a = this.getCache(sSortType, nColumn);
	var tBody = this.tBody;

	a.sort(f);
	if (this.descending) {
		a.reverse();
	}
	
	if (CommonSortObject.removeBeforeSort) {
		// remove from doc
		var nextSibling = tBody.nextSibling;
		var p = tBody.parentNode;
		p.removeChild(tBody);
	}

	// insert in the new order
	var l = a.length;
	for (var i = 0; i < l; i++){
		tBody.appendChild(a[i].element);
	}		

	if (CommonSortObject.removeBeforeSort) {
		// insert into doc
		p.insertBefore(tBody, nextSibling);
	}

	this.updateHeaderArrows();

	this.destroyCache(a);

	if (typeof this.onsort == "function")
		this.onsort();
};

CommonSortObject.prototype.asyncSort = function (nColumn, bDescending, sSortType) {
	var oThis = this;
	this._asyncsort = function () {
		oThis.sort(nColumn, bDescending, sSortType);
	};
	window.setTimeout(this._asyncsort, 1);
};

CommonSortObject.prototype.getCache = function (sType, nColumn) {
	if (!this.tBody) return [];
	var rows = this.tBody.rows;
	var l = rows.length;
	var a = new Array(l);
	var r;
	for (var i = 0; i < l; i++) {
		r = rows[i];
		a[i] = {
			value:		this.getRowValue(r, sType, nColumn),
			element:	r
		};
	};
	return a;
};

CommonSortObject.prototype.destroyCache = function (oArray) {
	var l = oArray.length;
	for (var i = 0; i < l; i++) {
		oArray[i].value = null;
		oArray[i].element = null;
		oArray[i] = null;
	}
};

CommonSortObject.prototype.getRowValue = function (oRow, sType, nColumn) {
	// if we have defined a custom getRowValue use that
	if (this._sortTypeInfo[sType] && this._sortTypeInfo[sType].getRowValue)
		return this._sortTypeInfo[sType].getRowValue(oRow, nColumn);

	var c = oRow.cells[nColumn];
	var s = CommonSortObject.getInnerText(c);
		
	return this.getValueFromString(s, sType);
};

CommonSortObject.getInnerText = function (oNode) {
	var s = "";
	var cs = oNode.childNodes;
	var l = cs.length;
	for (var i = 0; i < l; i++) {
        if(typeof cs[i].tagName == "undefined"){
        } else {
            var tagName = cs[i].tagName.toUpperCase();
            var id = cs[i].getAttribute("id");
            if(tagName == "SPAN" && id != null && id.indexOf("hidden") > -1){
                s = cs[i].innerHTML;
                break;
            }
        }
		switch (cs[i].nodeType) {
			case 1: //ELEMENT_NODE
				s += CommonSortObject.getInnerText(cs[i]);
				break;
			case 3:	//TEXT_NODE
				s += cs[i].nodeValue;
				break;
		}
	}
	return s;
};

CommonSortObject.prototype.getValueFromString = function (sText, sType) {
	if (this._sortTypeInfo[sType])
		return this._sortTypeInfo[sType].getValueFromString( sText );
	return sText;
};

CommonSortObject.prototype.getSortFunction = function (sType, nColumn) {
	if (this._sortTypeInfo[sType])
		return this._sortTypeInfo[sType].compare;
	return CommonSortObject.basicCompare;
};

CommonSortObject.prototype.destroy = function () {
	this.uninitHeader();
	var win = this.document.parentWindow;
	if (win && typeof win.detachEvent != "undefined") {	// only IE needs this
		win.detachEvent("onunload", this._onunload);
	}
	this._onunload = null;
	this.element = null;
	this.tHead = null;
	this.tBody = null;
	this.document = null;
	this._headerOnclick = null;
	this.sortTypes = null;
	this._asyncsort = null;
	this.onsort = null;
};

// Adds a sort type to all instance of CommonSortObject
// sType : String - the identifier of the sort type
// fGetValueFromString : function ( s : string ) : T - A function that takes a
//    string and casts it to a desired format. If left out the string is just
//    returned
// fCompareFunction : function ( n1 : T, n2 : T ) : Number - A normal JS sort
//    compare function. Takes two values and compares them. If left out less than,
//    <, compare is used
// fGetRowValue : function( oRow : HTMLTRElement, nColumn : int ) : T - A function
//    that takes the row and the column index and returns the value used to compare.
//    If left out then the innerText is first taken for the cell and then the
//    fGetValueFromString is used to convert that string the desired value and type

CommonSortObject.prototype.addSortType = function (sType, fGetValueFromString, fCompareFunction, fGetRowValue) {
	this._sortTypeInfo[sType] = {
		type:				sType,
		getValueFromString:	fGetValueFromString || CommonSortObject.idFunction,
		compare:			fCompareFunction || CommonSortObject.basicCompare,
		getRowValue:		fGetRowValue
	};
};

// this removes the sort type from all instances of CommonSortObject
CommonSortObject.prototype.removeSortType = function (sType) {
	delete this._sortTypeInfo[sType];
};

CommonSortObject.basicCompare = function compare(n1, n2) {
	var result = 0;
	if (n1.value < n2.value) {
		result = -1;
	} else if (n1.value > n2.value){
	    result = 1;
	} else {
		result = 0;
	}
	return result;
};

CommonSortObject.idFunction = function (x) {
	return x;
};

CommonSortObject.toUpperCase = function (s) {
	return s.toUpperCase();
};

CommonSortObject.toDate = function (s) {
	date = parseSimpleFormatDate(s);
	return date.valueOf();
};

CommonSortObject.toNumber = function (s) {
	return Number(parseDecimal(s));
}

// add sort types
CommonSortObject.prototype.addSortType("Number", CommonSortObject.toNumber);
CommonSortObject.prototype.addSortType("CaseInsensitiveString", CommonSortObject.toUpperCase);
CommonSortObject.prototype.addSortType("Date", CommonSortObject.toDate);
CommonSortObject.prototype.addSortType("String",CommonSortObject.idFunction,stringCompare);

//For defect 9136, 
//volume name need special sort: sort by prefix as String first; and then sort suffix as Number.
CommonSortObject.prototype.addSortType("VolumeName",CommonSortObject.idFunction,volNameCompare);


var volPrefix = "";
var volSuffix = "";
function volNameCompare(volName1, volName2){
	var volName1Suffix = "";
	var volName1Prefix = "";
	var volName2Suffix = "";
	var volName2Prefix = "";
	
	splitVolNameToPrefixSuffix(volName1);
	volName1Prefix = volPrefix;
	volName1Suffix = volSuffix;
	splitVolNameToPrefixSuffix(volName2);
	volName2Prefix = volPrefix;
	volName2Suffix = volSuffix;

	//prefix has higher weight than suffix.
	result = stringCompareAlgorithm(volName1Prefix,volName2Prefix);
	//if prefix is the same, we need to compare the suffix
	if(result == 0)
	{
		if(null == volName1Suffix || volName1Suffix.length == 0){
				result = -1;
			}else if(null == volName2Suffix || volName2Suffix.length == 0){
				result = 1;
			}else if(null!=volName1Suffix && volName1Suffix.length>0 
			  && null!=volName2Suffix && volName2Suffix.length>0){
			  	volName1Suffix = Number(volName1Suffix);
				volName2Suffix = Number(volName2Suffix);
				if(volName1Suffix < volName2Suffix){
					result = -1;
				}else if(volName1Suffix > volName2Suffix){
					result = 1;
				}else{
					result = 0;
				}
			}
	}
	return result;
}
function splitVolNameToPrefixSuffix(volName){
	var volNameStr = volName.value;
	volPrefix = "";
	volSuffix = "";
	if (volNameStr) {
		for (var i = volNameStr.length - 1; i >= 0; i--) {
			if (volNameStr.charAt(i) >= 0 && volNameStr.charAt(i) <= 9) {
				volSuffix = volNameStr.charAt(i) + volSuffix;
			} else {
				break;
			}
		}
		if (volSuffix.length > 0) {
			volPrefix = volNameStr.substring(0, volNameStr.lastIndexOf(volSuffix));
		}else{
			volPrefix = volNameStr;
		}
	}
}

function stringCompare(n1,n2)
{
	var result = 0;
	if(n1 == null)
	{
		result = -1;
	}
	else if(n2 == null)
	{
		result = 1;
	}else if(n1!=null && n2!=null)
	{
		str1 = n1.value;
		str2 = n2.value;
		result = stringCompareAlgorithm(str1,str2);
	}
	return result;
}
