/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
// ErrorCounters.js
var selectedBlades = new Array();
var selectedController3 = new Array();
var selectedController4 = new Array();
var selectedBay3ExternalPorts = new Array();
var selectedBay4ExternalPorts = new Array();

function stopErrorCounterWait()
{
	document.getElementById("tableFrame").style.display = "block";
	
	document.getElementById("waitDiv").style.display = "none";
}

function startErrorCounterWait()
{
	document.getElementById("tableFrame").style.display = "none";
	
	document.getElementById("waitDiv").style.display = "block";
}

function toggleResource(cell, image, array, id)
{
	// Inputs:
	// cell: the cell object that needs to be highlighted
	// image: the id of the image that represents the resource
	// array: the array (selectedBlades, selectedStorageModule1Disks, selectedStorageModule2Disks, selectedBay3ExternalPorts or selectedBay4ExternalPorts) that this resource belongs in
	// id: the resource ID (a zero indexed index into the category. ex, blade 3 would be represented by a 2 in the selectedBlades array)
	
	// Get the index of the id
	index = indexOf(id, array);
	
	if(index != -1)
	{
		// Here the resource is selected
		
		// Remove the item from the selected resources
		array.splice(index, 1);
		
		// Remove the border
		cell.className = "unSelectedResource";
		
		// Change the image alt tag
		// The alt tag will be composed of two strings: the resource name (stored in the title attribute) and a string indicating if the resource is not selected
		image = document.getElementById("resourceImage" + (id + getResourceOffset(array)) );
		image.alt = image.title + " " + document.getElementById(namespace_g+ "notSelected").value;
	}
	else
	{
		// Here the resource is not selected
		
		// Add the resource to the selected resources
		array.push(id);
			
		// Add a border
		cell.className = "selectedResource";
		
		// Change the image alt tag
		// The alt tag will be composed of two strings: the resource name (stored in the title attribute) and a string indicating if the resource is selected
		image = document.getElementById("resourceImage" + (id + getResourceOffset(array)) );
		image.alt = image.title + " " + document.getElementById(namespace_g +"selected").value;
		
	}
	
	// Enable or disable control buttons
	if(getNumSelectedResources() == 0)
	{
		// Disable
		document.getElementById("viewErrorCountersButton").disabled = true;
		document.getElementById("resetSelectionsButton").disabled = true;
	}
	else
	{
		// Enable
		document.getElementById("viewErrorCountersButton").disabled = false;
		document.getElementById("resetSelectionsButton").disabled = false;
	}
}

function getResourceOffset(array)
{
	if(array == selectedBlades)
		return 0;
	if(array == selectedController3)
		return 6;
	if(array == selectedController4)
		return 15;
	if(array == selectedBay3ExternalPorts)
		return 7;
	if(array == selectedBay4ExternalPorts)
		return 11;
		
}

function indexOf(element, array)
{
	for(x = 0; x < array.length; x++)
		if(array[x] == element)
			return x;
			
	return -1;
}

function getNumSelectedResources()
{
	return selectedBlades.length + selectedController3.length +  selectedController4.length + selectedBay3ExternalPorts.length + selectedBay4ExternalPorts.length;
}


function viewErrorCounters()
{
	// Build the query string of selected devices
	var queryString = "";
	
	queryString += buildQueryString("blades", selectedBlades);
	queryString += buildQueryString("storageController3", selectedController3);
	queryString += buildQueryString("storageController4", selectedController4);
	queryString += buildQueryString("bay3ExternalPorts", selectedBay3ExternalPorts);
	queryString += buildQueryString("bay4ExternalPorts", selectedBay4ExternalPorts);
	
	// The query string will always end with an extra &, chop it off
	queryString = queryString.substring(0, queryString.length - 1);
	
	// Show the wait section while loading	
	startWait();
	document.getElementById('tableFrame').src = document.global_context_path + '/faces/page/rbce/errorcounters/ErrorCountersTable.jsp?' + queryString;
}

function buildQueryString(name, array)
{
	var result = "";
	
	for(x = 0; x < array.length; x++)
	{
		result += name + "=" + array[x] + "&";	
	}
	
	return result;
}

function resetSelections()
{
	// Unselecting all selected resources
	unselectAll(selectedBlades);
	unselectAll(selectedController3);
	unselectAll(selectedController4);
	unselectAll(selectedBay3ExternalPorts);
	unselectAll(selectedBay4ExternalPorts);
	
	// Resetting all arrays to empty
	selectedBlades = new Array();
	selectedController3 = new Array();
	selectedController4 = new Array();
	selectedBay3ExternalPorts = new Array();
	selectedBay4ExternalPorts = new Array();
	
	// Disable control buttons
	document.getElementById("viewErrorCountersButton").disabled = true;
	document.getElementById("resetSelectionsButton").disabled = true;
}

function unselectAll(array)
{
	for(x = 0; x < array.length; x++)
	{
		image = document.getElementById("resourceImage" + (array[x] + getResourceOffset(array)) );
		image.alt = image.title + " " + document.getElementById(namespace_g+ "notSelected").value;
		
		cell = document.getElementById("resourceCell" + (array[x] + getResourceOffset(array)) );
		cell.className = "unSelectedResource";
	}
}

function matchBladeNumberAlt(bladeImageID, numberImageID)
{
	// So that the blade number alt text matches the blade image alt text
	// This is called after the blade image alt text changes due to a select/deselect
	document.getElementById(numberImageID).alt = document.getElementById(bladeImageID).alt;
}

function resizeFrameHeight(frameID, frameName)
{
	frameObject = document.getElementById(frameID);
	
	// Reset it to a small value so that the height is always increased
	frameObject.style.height = 10;
	
	frameDocument = window.frames[frameName].document;
	
	frameBody = frameDocument.body;

	//  Height
	oHeight = 0;
	sHeight = 0;
	
	if(frameBody.offsetHeight)
		oHeight = frameBody.offsetHeight;
	
	if(frameBody.scrollHeight)
		sHeight = frameBody.scrollHeight;
		
	// Set the height to the bigger one
	newHeight = Math.max(oHeight, sHeight);
	
	frameObject.style.height = newHeight;
}
