/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
// ManageFabric.js

// Blade Colors
bladeColor0 = "#f1cb78";
bladeColor1 = "#a7bbdd";
bladeColor2 = "#db8f8f";
bladeColor3 = "#b791ef";
bladeColor4 = "#a3d9a9";
bladeColor5 = "#bdbdbb";

// Mapping images
numberImage0 = document.global_context_path + "/images/numberImages/numberImage1.gif";
numberImage1 = document.global_context_path + "/images/numberImages/numberImage2.gif";
numberImage2 = document.global_context_path + "/images/numberImages/numberImage3.gif";
numberImage3 = document.global_context_path + "/images/numberImages/numberImage4.gif";
numberImage4 = document.global_context_path + "/images/numberImages/numberImage5.gif";
numberImage5 = document.global_context_path + "/images/numberImages/numberImage6.gif";

no_numberImage = document.global_context_path + "/images/fabric/no_number.gif";

// The HTML table cell object of the selected blade
var selectedBladeNum;

var hasBeenModified = false;

// Message pattern
var manageFabricRAIDSubsystemPattern = "";
var manageFabricIOBayPattern = "";
var manageFabricIOBayLabelPattern = "";
var manageFabricEnabledPattern = "";
var manageFabricDisabledPattern = "";

function init()
{
    manageFabricRAIDSubsystemPattern = document.getElementById("manageFabricRAIDSubsystemHiddenMsg").innerHTML;
	manageFabricIOBayPattern = document.getElementById("manageFabricIOBayHiddenMsg").innerHTML;
	manageFabricEnabledPattern = document.getElementById("manageFabricEnabledHiddenMsg").innerHTML;
	manageFabricDisabledPattern = document.getElementById("manageFabricDisabledHiddenMsg").innerHTML;
	manageFabricIOBayLabelPattern = document.getElementById("manageFabricIOBayHiddenLabel").innerHTML;
}

function getHasBeenModified()
{
	return hasBeenModified;
}

function changeSelectedBlades(bladeNum)
{		
	// Unselect previous	
	if(selectedBladeNum != null)
	{
		document.getElementById("bladeCell" + selectedBladeNum).className = "unmappedStorage"
	}
	
	// Select new
	selectedBladeNum = bladeNum;
	document.getElementById("bladeCell" + bladeNum).className = "mappedStorage";
	document.getElementById("bladeCell" + bladeNum).style.borderColor = eval("bladeColor" + bladeNum);
}


function selectMapped(bladeNum)
{
	bladePortIDArrays = bladeIDs[bladeNum];

	// Highlight all the mapped ports
	for(x = 0; x < bladePortIDArrays.length; x++)
	{
		for(y = 0; y < bladePortIDArrays[x].length; y++)
		{
			if(document.getElementById(bladePortIDArrays[x][y]))
			{						
				document.getElementById(bladePortIDArrays[x][y]).className="mappedStorage";
				document.getElementById(bladePortIDArrays[x][y]).style.borderColor = eval("bladeColor" + bladeNum);
			}
		}
	}	
}

function deselectMapped(bladeNum)
{
	bladePortIDArrays = bladeIDs[bladeNum];
	
	// Highlight all the mapped ports
	for(x = 0; x < bladePortIDArrays.length; x++)
	{
		for(y = 0; y < bladePortIDArrays[x].length; y++)
		{	
			if(document.getElementById(bladePortIDArrays[x][y]))
			{
				document.getElementById(bladePortIDArrays[x][y]).className="unmappedStorage";
			}	
		}
	}
}

function toggleMapping(bladeNum, portID)
{
	// Set the modified flag
	hasBeenModified = true;

	bladePortIDArrays = bladeIDs[bladeNum];
	
	// See if this port is already mapped to the blade
	alreadyMapped = false;
	
	for(x = 0; x < bladePortIDArrays.length; x++)
	{		
		index = indexOf(portID, bladePortIDArrays[x]);
		
		if(index != -1)
		{
			alreadyMapped = true;
		}
	}
	
	if(alreadyMapped)
	{
		// if alreadyMapped, we need to switch the mapping to off
		// this mean to remove the port ID from all the bladeArrays
		
		// Apply the change locally
		unmap(bladeNum, portID);
		
		// Apply the change on the server
		sendUnmapRequest(bladeNum, portID);
	}
	else
	{
		// Apply the change locally
		map(bladeNum, portID);
		
		// Apply the change on the server
		sendMapRequest(bladeNum, portID);
	}
	
	// Make the blade the selected balde
	changeSelectedBlades(bladeNum);
}

function map(bladeNum, portID)
{
	bladePortIDArrays = bladeIDs[bladeNum];
	
	// alert("map");
	// Make sure that this portID appears in all blade arrays
	
	for(x = 0; x < bladePortIDArrays.length; x++)
	{
		if(indexOf(portID, bladePortIDArrays[x]) == -1)
		{
			bladePortIDArrays[x].push(portID);
		}
	}
	
	// Change the mapping icon
	// alert(portID + bladeNum);
	if(document.getElementById(portID + "" + bladeNum))
	{
		document.getElementById(portID + "" + bladeNum).src = eval("numberImage" + bladeNum);
		changeMapTitle(bladeNum, portID, true);
	}
}

function unmap(bladeNum, portID)
{
	bladePortIDArrays = bladeIDs[bladeNum];
	
	// alert("unmap");
	
	// Make sure that this portID appears in none of the blade arrays
	
	for(x = 0; x < bladePortIDArrays.length; x++)
	{
		index = indexOf(portID, bladePortIDArrays[x]);

		if(index != -1)
		{
			// Remove if its found
			bladePortIDArrays[x].splice(index, 1);
		}
	}
	
	// Change the mapping icon
	// alert(portID + bladeNum);
	if(document.getElementById(portID + "" + bladeNum))
	{
		document.getElementById(portID + "" + bladeNum).src = no_numberImage;
		changeMapTitle(bladeNum, portID, false);
	}
}

function changeMapTitle(bladeNum, portID, isEnabled) {
	var bay3portobj = document.getElementById("span" + portID + "_3");
	var bay4portobj = document.getElementById("span" + portID + "_4");
	var stateMessage = "";
	if(isEnabled)
	{
		stateMessage = manageFabricEnabledPattern;
	}
	else
	{
		stateMessage = manageFabricDisabledPattern;
	}
	var nlsMessage = "";
	if (bay3portobj) {
		var port3Index = bay3portobj.innerHTML;
		nlsMessage = composeMessage(manageFabricIOBayPattern,{BLADE_BAY_NUM:(bladeNum+1),EXTERNAL_PORT_NUM:port3Index,IO_PORT_NUM:3,SELECT_STATE:stateMessage});
	} else if (bay4portobj) {
		var port4Index = bay4portobj.innerHTML;
		nlsMessage = composeMessage(manageFabricIOBayPattern,{BLADE_BAY_NUM:(bladeNum+1),EXTERNAL_PORT_NUM:port4Index,IO_PORT_NUM:4,SELECT_STATE:stateMessage});
	} else {
		nlsMessage = composeMessage(manageFabricRAIDSubsystemPattern,{BLADE_BAY_NUM:(bladeNum+1),SELECT_STATE:stateMessage});
	}
	document.getElementById(portID + "" + bladeNum).title = nlsMessage;
	document.getElementById(portID + "" + bladeNum).alt = nlsMessage;
}

function loadMapTitle(bladeNum, portID) {
	var component = document.getElementById(portID + "" + bladeNum);
	if(component && (!component.alt || (component.alt && component.alt=="")))
	{
		changeMapTitle(bladeNum, portID, false);
	}
}

function loadImgTitle(component,portID) {
	var bay3portobj = document.getElementById("span" + portID + "_3");
	var bay4portobj = document.getElementById("span" + portID + "_4");
	
	if (bay3portobj) {
		var port3Index = bay3portobj.innerHTML;
		component.title = composeMessage(manageFabricIOBayLabelPattern,{EXTERNAL_PORT_NUM:port3Index,IO_PORT_NUM:3});
		component.alt = composeMessage(manageFabricIOBayLabelPattern,{EXTERNAL_PORT_NUM:port3Index,IO_PORT_NUM:3});
	} else if (bay4portobj) {
		var port4Index = bay4portobj.innerHTML;
		component.title = composeMessage(manageFabricIOBayLabelPattern,{EXTERNAL_PORT_NUM:port4Index,IO_PORT_NUM:4});
		component.alt = composeMessage(manageFabricIOBayLabelPattern,{EXTERNAL_PORT_NUM:port4Index,IO_PORT_NUM:4});
	}
}

function changeMapping(portID)
{
	// This function is called if the icon for a prot is clicked
	
	// Make sure that we have a selected blade
	if(selectedBladeNum == null)
	{
		return;
	}
	
	toggleMapping(selectedBladeNum, portID);
}

function selectAll(portList)
{
	// Make sure that we have a selected blade
	if(selectedBladeNum == null)
	{
		return;
	}
	
	// Set the modified flag
	hasBeenModified = true;
	
	// PortList will be an array of port IDs that are to be mapped
	numMapped = getNumMapped(portList, selectedBladeNum);
	
	// Check if all are alrady mapped
	if(numMapped == portList.length)
	{
		// If all are already mapped, then deselect all
		
		for(i = 0; i < portList.length; i++)
		{
			// Apply the change locally
			unmap(selectedBladeNum, portList[i]);
			
			// Apply the change on the server
			sendUnmapRequest(selectedBladeNum, portList[i]);
		}
	}
	else
	{
		// If not all are mapped, then map all
		
		for(i = 0; i < portList.length; i++)
		{
			// Apply the change locally
			map(selectedBladeNum, portList[i]);
			
			// Apply the change on the server
			sendMapRequest(selectedBladeNum, portList[i]);
		}
	}
}

function getNumMapped(portList, bladeNum)
{
	// Find how many of these ports are mapped to this blade
	
	counter = 0;
	
	bladePortIDArrays = bladeIDs[bladeNum];
	
	for(x = 0; x < bladePortIDArrays.length; x++)
	{
		for(y = 0; y < portList.length; y++)
		{			
			if(indexOf(portList[y], bladePortIDArrays[x]) != -1)
			{
				counter++;
			}
		}
	}
	
	// Since in this version ports are mapped to all blade ports (one for each switch)
	return counter / bladePortIDArrays.length;
}

function clear()
{
	// Make sure to unmap all locally
	for(i = 0; i < bladeIDs.length; i++)
	{
		for(j = 0; j < bladeIDs[i].length; j++)
		{
			while(bladeIDs[i][j].length > 0)
			{
				bladePortIndex = i;
				mappedPortID = bladeIDs[i][j].pop();
				
				unmap(bladePortIndex, mappedPortID);
			}
		}	
	}
	
	// Clear on the server
	sendClearRequest();
}


//////////////////////////////////////// AJAX FUNCTIONS ////////////////////////////////////////////////////////

function sendMapRequest(bladeNum, portID)
{
	bladePortIDArrays = bladePortNames[bladeNum];
	
	var mappings = "";
	
	for(z = 0; z < bladePortIDArrays.length; z++)
	{
		bladePort = bladePortIDArrays[z];
		
		mappings += "map=" + bladePort + "mapsto" + portID + "&";
	}
	
	requestObject = makeHttpRequestObject();

	requestObject.open("GET", editorServletPath + '?action=map&' + mappings, true);
		
	requestObject.send(null);
}

function sendUnmapRequest(bladeNum, portID)
{
	bladePortIDArrays = bladePortNames[bladeNum];
	
	var mappings = "";
	
	for(z = 0; z < bladePortIDArrays.length; z++)
	{
		bladePort = bladePortIDArrays[z];
		
		mappings += "unmap=" + bladePort + "mapsto" + portID + "&";
	}
	
	requestObject = makeHttpRequestObject();

	requestObject.open("GET", editorServletPath + '?action=unmap&' + mappings, true);
		
	requestObject.send(null);
}

function sendClearRequest()
{
	requestObject = makeHttpRequestObject();

	requestObject.open("GET", editorServletPath + '?action=clear', true);
		
	requestObject.send(null);
}

function makeHttpRequestObject() 
{
	var xmlreq = false;

	if (window.XMLHttpRequest) 
	{
    	// Create XMLHttpRequest object in non-Microsoft browsers
    	xmlreq = new XMLHttpRequest();
	}
	else if (window.ActiveXObject) 
			{			
			    // Create XMLHttpRequest via MS ActiveX
		      	xmlreq = new ActiveXObject("Microsoft.XMLHTTP");
		  	}
  	
	return xmlreq;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////




///////////////////////////////////// HELPER FUNCTIONS ////////////////////////////////////////////////////////

function getMappings()
{
	// A function that can be used to get the current state of mappings
	
	var result = new Array();
	
	for(x = 0; x < bladeIDs.length; x++)
	{
		for(y = 0; y < bladeIDs[x].length; y++)
		{
			for(z = 0; z < bladeIDs[x][y].length; z++)
			{
				bladePort = bladePortNames[x][y];
				mappedPort = bladeIDs[x][y][z];
				
				result.push(new Array(bladePort, mappedPort));
			}
		}	
	}

	return result;
}

function indexOf(element, array)
{
	for(q = 0; q < array.length; q++)
		if(array[q] == element)
			return q;
			
	return -1;
}

function composeMessage(pattern,kvHash)
{
	if(pattern==null || kvHash==null)
	{
		return "";
	}
	for(key in kvHash)
	{
		pattern = pattern.replace(new RegExp(key,"g"), kvHash[key]);
	}			
	return pattern;
}
