/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
// AlertsMenu.js

// ------------------------------------------------------ VARIABLES --------------------------------
var selectedMenuItem;
var menuIndex = 0;//menuIndex is used to record which one is selected
var currentIndex = 0;//currentIndex is used to track which alert code is currently on focus
var isMenuShowing = false;
var menuPresent = true;

var MAX_ITEMS = 6;
var MIN_WIDTH = 150;

//two style of alert code
var selectedItemStyle = "selectedMenuItem";
var unSelectedItemStyle = "unselectedMenuItem";

//used to record which item is focus, mainly used for doFocus() function
var focusItem = null;
var lastFocusItem = null;

var KEY_UP = 38;
var KEY_DOWN = 40;
var KEY_ENTER = 13;
var KEY_ESCAPE = 27;
var KEY_SPACE = 32;

var comboBoxLabelPattern = "";
var ALERT_STATUS_TEXT = "ALERT_STATUS_TEXT";
var ALERT_CODE = "ALERT_CODE";

// ------------------------------------------------------ HELPER FUNCTIONS --------------------------------
function getAbsoulteX(htmlElement)
{
	absoluteX = 0;
	
	while(htmlElement) 
	{
		absoluteX += htmlElement.offsetLeft;
		htmlElement = htmlElement.offsetParent;
	}
	
	return absoluteX;
}

function getAbsoulteY(htmlElement)
{
	absoluteY = 0;
	while(htmlElement) 
	{
		absoluteY += htmlElement.offsetTop;
		htmlElement = htmlElement.offsetParent;
	}
	
	return absoluteY;
}

function stopPropagation(thisEvent)
{
	// For IE
	if(!thisEvent)
		thisEvent = window.event;
		
	thisEvent.cancelBubble = true;
		
	// For Mozilla
	if(thisEvent.stopPropagation)
		thisEvent.stopPropagation();
}

function registerEventHandler(event,target,handler,useCapture)
{
	if (target && document.addEventListener) { 
		// Register capturing event handlers
		target.addEventListener(event, handler, useCapture);
	}
	// IE 5+ Event Model
	else if (target && document.attachEvent) {
		target.attachEvent("on" + event, handler);
	}
}

function isOutsideMenu(menuObject, e)
{
	if (window.event)
		e = window.event;
		
	var x = e.clientX;
	var y = e.clientY;

	var top = getAbsoulteY(menuObject);
	var left = getAbsoulteX(menuObject);
	
	if(x < left || x > (left + menuObject.offsetWidth) || y < top || y > (top + menuObject.offsetHeight))
	{
		return true;
	}
	
	return false;
}

function doFocus()
{
	if(focusItem != null)
	{
		if(lastFocusItem && lastFocusItem.className == selectedItemStyle)
		{
			lastFocusItem.className = unSelectedItemStyle;
		}
		if(focusItem && focusItem.className == unSelectedItemStyle)
		{
			focusItem.className = selectedItemStyle;
		}
		setTimeout(function(){focusItem.focus();lastFocusItem = focusItem;focusItem = null;},0);
	}
	
}

function composeMessage(pattern,kvHash)
{
	if(pattern==null || kvHash==null)
	{
		return "";
	}
	for(key in kvHash)
	{
		pattern = pattern.replace(new RegExp(key,"g"), kvHash[key]);
	}			
	return pattern;
}

function changeComboBoxDesc()
{
	//change the combobox description label content
	var alertData = eval("alert" + menuIndex);
	document.getElementById("comboboxDivLabel").getElementsByTagName("span")[0].innerHTML = composeMessage(comboBoxLabelPattern,{ALERT_STATUS_TEXT:alertData[2],ALERT_CODE:alertData[4]});
}
// ------------------------------------------------------ EVENT FUNCTIONS --------------------------------

function focusCombo(event,menuId,comboId)
{
	event = event || window.event;
	stopPropagation(event);
	var key = event.keyCode;
	switch(key)
	{
		case KEY_ESCAPE://Just close it, set focus to combo
			hideMenu();
			focusItem = document.getElementById(comboId);
			break;
		case KEY_ENTER://open item and set the focus to the old one
		case KEY_DOWN:
		case KEY_SPACE:
			showMenu();
			var selectedAlertCode = eval("alert" + menuIndex)[4];
			var items = getMenuItems(document.getElementById(menuId));
			doFocusItem(items,selectedAlertCode);
			break;
	}
}

function clearMenuItemStyle(menu)
{
	var menuItems = getMenuItems(menu);
	for(var i=0;i<menuItems.length;i++)
	{
		menuItems[i].className = unSelectedItemStyle;
	}
}

function doFocusItem(menuItems,value)
{
	for(var i=0;i<menuItems.length;i++)
	{
		if(menuItems[i].innerHTML.indexOf(value)!= -1)
		{
			focusItem = menuItems[i];
			doFocus();
			break;
		}
		
	}
}

function getMenuItems(menu)
{
	var items = new Array();
	var elements = menu.getElementsByTagName("div");
	for(var i=0;i<elements.length;i++)
	{
		if(elements[i].getAttribute("role") == "listitem")
		{
			items[items.length] = elements[i];
		}
	}
	
	return items;
}

function doComboNav(event,menuId,comboId,inputId)
{
	event = event || window.event;
	stopPropagation(event);
	var menu = document.getElementById(menuId);
	var menuItems = getMenuItems(menu);
	var menuLength = getMenuItems(menu).length;
	switch(event.keyCode)
	{
		case KEY_UP://up
			currentIndex = (currentIndex-1+menuLength)%menuLength;
			focusItem = menuItems[currentIndex];
			doFocus();
			break;
		case KEY_DOWN://down
			currentIndex = (currentIndex+1)%menuLength;
			focusItem = menuItems[currentIndex];
			doFocus();	
			break;
		case KEY_ENTER://enter
		case KEY_SPACE://space
			chooseMenuItem(currentIndex,menu,inputId);
			focusItem = document.getElementById(comboId);
			doFocus();
			//change combobox description label
			changeComboBoxDesc();
			break;
		case KEY_ESCAPE: //escape
			hideMenu();
			focusItem = document.getElementById(comboId);
			doFocus();
			break;
	}
}

function openCombo(event,menuId,comboId)
{
	event = event || window.event;
	stopPropagation(event);
	if(event.keyCode == KEY_ENTER)
	{
		if(isMenuShowing)
		{
			hideMenu();
			//set the focus on menu
			focusItem = document.getElementById(comboId);
			doFocus();
		}
		else
		{
			showMenu();
			//focus on the current alert code item
			var selectedAlertCode = eval("alert" + menuIndex)[4];
			var items = getMenuItems(document.getElementById(menuId));
			doFocusItem(items,selectedAlertCode);
		}
	}
}

function arrowClicked(e)
{
	stopPropagation(e);
	
	if(isMenuShowing)
	{
		hideMenu();
	}
	else
	{
		showMenu();
	}
}

function bodyClicked(e)
{
	// Kick out if the menu is not there
	if(!menuPresent)
	{
		return;
	}
	
	// Ignore the arrow clicks
	if(! isOutsideMenu(document.getElementById("arrowImage"), e))
	{
		return;
	}
	
	// Check if we need to hide the menu
	if(isOutsideMenu(document.getElementById("menuDiv"), e))
	{
		hideMenu();
	}
	
	// Because JWL redraws components
	// Add the onscroll for the alerts div on every click
	var alertsDiv = document.getElementById(namespace_g + "mainForm:informationTabbedPanel:alertsTab");

	alertsDiv.onscroll = positionMenu;
}

// ------------------------------------------------------ MENU DISPLAY FUNCTIONS --------------------------------

function showMenu(e)
{	
	// Select the current item
	if(selectedMenuItem)
	{
		selectedMenuItem.className = "selectedMenuItem"; 
	}
	
	positionMenu();
	
	var menu = document.getElementById("menuDiv");
	
	// Show the menu
	menu.style.display = "block";
	menu.style.visibility = "visible";
	
	isMenuShowing = true;
}

function positionMenu()
{
	// Position the menu
	var headerTable = document.getElementById("menuHeaderTable");
	var menu = document.getElementById("menuDiv");
	
	var alertsDiv = document.getElementById(namespace_g + "mainForm:informationTabbedPanel:alertsTab");
		
	menu.style.top = getAbsoulteY(headerTable) + headerTable.offsetHeight - alertsDiv.scrollTop;
	menu.style.left = getAbsoulteX(headerTable);
}

function hideMenu()
{	
	var menu = document.getElementById("menuDiv");
	
	menu.style.display = "none";
	menu.style.visibility = "visible";	
	clearMenuItemStyle(menu);
	
	isMenuShowing = false;	
}

// ------------------------------------------------------ MENU ITEM EVENT FUNCTIONS --------------------------------

function menuItemOnMouseOver(menuItemObject)
{
	// Change the style of the selected item, but dont change the value of the variable
	if(selectedMenuItem)
	{
		selectedMenuItem.className = "unselectedMenuItem";
	}
	
	menuItemObject.className="selectedMenuItem";
}

function menuItemOnMouseOut(menuItemObject)
{
	menuItemObject.className="unselectedMenuItem";
}


function menuItemOnMouseClick(menuItemObject, clickedIndex)
{
	menuItemObject.className="selectedMenuItem";
	
	// Hide the menu
	document.getElementById("menuDiv").style.display = "none";
	
	selectedMenuItem = menuItemObject;
	
	// Set the value of the selected object
	document.getElementById("menuHeaderTableValueSpan").innerHTML = selectedMenuItem.innerHTML
	
	// Update the rest of the screen
	menuIndex = clickedIndex;
	changeAlert();
	initAlertsBtns(alertData,menuIndex);
	
	//change combobox description label
	changeComboBoxDesc();
}

function chooseMenuItem(clickedIndex,menu,inputId)
{
	
	// Hide the menu
	document.getElementById("menuDiv").style.display = "none";
	
	selectedMenuItem = getMenuItems(menu)[clickedIndex];
	
	// Set the value of the selected object
	document.getElementById(inputId).innerHTML = selectedMenuItem.getElementsByTagName("span")[0].innerHTML;
	
	// Update the rest of the screen
	menuIndex = clickedIndex;
	changeAlert();
	initAlertsBtns(alertData,menuIndex);
}

function changeAlert()
{
	alertData = eval("alert" + menuIndex);
	
	// Set all the fields
	document.getElementById("selectedAlertId").value = alertData[0];
	// document.getElementById(namespace_g + "mainForm:alertStatusImage").src = alertData[1]; // Not currently in use
	// document.getElementById(namespace_g + "mainForm:alertStatusText").innerHTML = alertData[2]; // Not currently in use
	document.getElementById(namespace_g + "mainForm:alertTimeStamp").innerHTML = alertData[3];
	document.getElementById(namespace_g + "mainForm:alertCode").innerHTML = alertData[4];
	document.getElementById(namespace_g + "mainForm:description").innerHTML = alertData[5];
	// document.getElementById(namespace_g + "mainForm:instructions").innerHTML = alertData[6]; // Not currently in use
}

// ------------------------------------------------------ INTIALIZE THE MENU --------------------------------

function initializeMenu()
{
	// If the menu has not been displaed
	if(!document.getElementById("menuDiv")
	|| !document.getElementById("menuTable") 
	|| !document.getElementById("menuHeaderTableValueSpan") 
	|| !document.getElementById("menuHeaderTableValueCell"))
	{
		menuPresent = false;
		
		// Cannot initialize
		return;
	}
	
	comboBoxLabelPattern = document.getElementById("comboBoxLabelHiddenMsg").innerHTML;
	
	var menu = document.getElementById("menuDiv");
	var menuTable = document.getElementById("menuTable");

	// Calculate menu height
	var height = 0;
	
	for(x = 0; x < MAX_ITEMS; x++)
	{
		if(x < menuTable.rows.length)
		{
			height += menuTable.rows[x].offsetHeight;
		}
	}
	
	// Check if we need a vertical scroll bar
	if(menuTable.rows.length > MAX_ITEMS)
	{
		menu.style.overflowY = "scroll";
	}
	else
	{
		menu.style.overflowY = "hidden";
	}
	
	menu.style.height = height;
	
	// Calculate menu width
	var width = 0;
	
	for(x = 0; x < menuTable.rows.length; x++)
	{		
		width = Math.max(width, menuTable.rows[x].offsetWidth);
	}
	
	// Check that we are not shorter than the minimum
	if(width < MIN_WIDTH)
	{
		width = MIN_WIDTH;
		menuTable.style.width = MIN_WIDTH;
	}
	
	menu.style.width = width + 25;
	
	// Adjust to be flush
	offset = 25;
	
	while(menu.scrollWidth > width)
	{
		offset--;
		
		menu.style.width = width + offset;
	}
	
	document.getElementById("menuHeaderTableValueSpan").style.width = width;
	document.getElementById("menuHeaderTableValueCell").style.width = width;
	setupInitialAlert();
	
	
	
}

// ------------------------------------------------------ select default alert item--------------------------------

function setupInitialAlert(){
	var defaultAlertID = parent.document.getElementById("defaultSelectedAlertID").value;
	
	var menuTable = document.getElementById("menuTable");
	
	// item length stand for the alert's length
	var menuItemLength = menuTable.rows.length;
	
	if(defaultAlertID!=null && defaultAlertID!="" && menuItemLength>0){
	
		var alertIndex = -1;
		
		// search the index of the alerts will be selected
		for(var i=0;i<menuItemLength;i++){
			var data = null;
			try{
			
				data = eval("alert" + i); 
			
			}catch(error){
			}
			
			if(data==null) 
				break;
			
			if(defaultAlertID==data[0]){
			
				alertIndex = i;
				
				break;
			}
		}
		
		if(alertIndex>=0){
			// match the default selection
			var tdObj = menuTable.rows[alertIndex].getElementsByTagName("div")[0];
			
			menuItemOnMouseClick(tdObj,alertIndex);
			
			if(alertIndex+1>MAX_ITEMS)
			{
			  	scrollMenu((alertIndex+1)/MAX_ITEMS);
			}
			
			// get the alert tab to be selected
			ODCRegistry.getClientControl('informationTabbedPanel').restoreUIState(namespace_g + "mainForm:informationTabbedPanel:alertsTab");
			// clean the default value
			parent.document.getElementById("defaultSelectedAlertID").value="";
			
		}
	}
		alertIndex = (alertIndex>=0)?alertIndex:0;
		try{
			data = eval("alert" + alertIndex); 
			initAlertsBtns(data,alertIndex);
		}catch(error){
		}
}

var GUI_Alert_Type_Clearable = 0;
var GUI_Alert_Type_Maskable = 1;
var GUI_Alert_Type_Unknown = 2;

function initAlertsBtns(alertData,type){
	var type = alertData[7];
	var masked = alertData[8];
	if(type==GUI_Alert_Type_Clearable){
		document.getElementById(namespace_g + "mainForm:clearBtn").disabled = false;	
	}else{
		document.getElementById(namespace_g + "mainForm:clearBtn").disabled = true;		
	}
	
	if(masked=="true"){
		document.getElementById(namespace_g + "mainForm:lightPathBtn").disabled = false;		
		document.getElementById(namespace_g + "mainForm:disLightPathBtn").disabled = true;	
		document.getElementById(namespace_g + "mainForm:lightPathBtn").style.display="";		
		document.getElementById(namespace_g + "mainForm:disLightPathBtn").style.display="none";	
	}else{
		document.getElementById(namespace_g + "mainForm:lightPathBtn").disabled = true;		
		document.getElementById(namespace_g + "mainForm:disLightPathBtn").disabled = false;	
		document.getElementById(namespace_g + "mainForm:lightPathBtn").style.display="none";		
		document.getElementById(namespace_g + "mainForm:disLightPathBtn").style.display="";			
	}
}

function scrollMenu(size){

	var menu = document.getElementById("menuDiv");
	var menuTable = document.getElementById("menuTable");	
	
	// set the scrolling size
	menu.scrollTop = size*menuTable.rows[0].offsetHeight*MAX_ITEMS;
	
	
}
